/*****************************************************************************
Author: 		Dave Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Date: 			1/29/09
Program: 		object_id_generator.ms
Description: 	This script will assign object ids to all or selected objects
******************************************************************************/

try (destroyDialog id_generator_rollout) catch()


struct ObjectIdGenerator
(
    /**************************************************************************
    Sorts items by name
    **************************************************************************/
    fn sort_by_name element_one element_two =
	(
		case of
		(
			(element_one.name < element_two.name): -1
			(element_one.name > element_two.name): 1
			default: 0
		) --end case
	), --end fn
    
    /**************************************************************************
    Returns a list of objects
    **************************************************************************/
    fn get_objects selection_only =
    (
        if selection_only then
            object_list = selection as array
        else
            object_list = objects as array
        
        qsort object_list this.sort_by_name
        object_list --return value
    ), --end fn
    
    /**************************************************************************
    Returns a list of all scene material names sorted
    **************************************************************************/
    fn get_material_names =
    (
        local names = for mat in sceneMaterials collect mat.name
        sort names --return value
    ), --end fn
    
    /**************************************************************************
    Assigns ids based on the material assigned to the object (ie if objects
    A, B and C all have the same material name then they will all get the same
    id value)
    **************************************************************************/
    fn set_ids_by_material object_list =
    (
        local material_names = this.get_material_names()
        
        for current in object_list do
        (
            if current.material != undefined do
            (
                index = findItem material_names current.material.name
                
                if index == 0 do
                (
                    index = material_names.count + 1
                    append material_names current.material.name
                ) --end else
                
                current.gbufferChannel = index
            ) --end if
        ) --end for
    ), --end fn
    
    /**************************************************************************
    Assigns a unique id for every object in the list
    **************************************************************************/
    fn set_ids_by_object object_list =
    (
        local counter = 1
        
        for current in object_list do
        (
            current.gbufferChannel = counter
            counter += 1
        ) --end fn
    ), --end fn
    
    /**************************************************************************
    Assigns the user id value to all objects in the list
    **************************************************************************/
    fn set_ids_by_user object_list id_value =
    (
        for current in object_list do
            current.gbufferChannel = id_value
    ), --end fn
    
    /**************************************************************************
    Offsets the current id value by the offset value
    **************************************************************************/
    fn offset_ids object_list offset_value =
    (
        for current in object_list do
            current.gbufferChannel += offset_value
    ), --end fn
    
    /**************************************************************************
    Swaps the 'from_value' id to the 'to_value' id for every object in the
    list 'object_list'
    **************************************************************************/
    fn swap_ids object_list from_value to_value =
    (
        for current in object_list do
        (
            current_id = current.gbufferChannel
            
            if current_id == from_value do
                current.gbufferChannel = to_value
            
            if current_id == to_value do
                current.gbufferChannel = from_value
        ) --end for
    ) --end fn
) --end struct


/**************************************************************************
UI creation
**************************************************************************/
( --local scope
    global id_generator_rollout
    local id_generator_ini = getDir #plugcfg + "\\ObjectIDGenerator.ini"
	local id_generator_position
    
    rollout id_generator_rollout "Object ID Generator"
    (
        local id_generator = ObjectIdGenerator()
        
        groupbox option_group "Options" pos:[5,5] width:175 height:40
            checkbox selection_checkbox "Selection Only" pos:[15,25] checked:true
            
        groupbox generate_group "Generate" pos:[5,50] width:175 height:95
            button id_material_button "by Material" pos:[15,70] width:155
            button id_object_button "by Object" pos:[15,95] width:155
            button id_user_button "User Defined" pos:[15,120] width:95 height:20
            spinner user_spinner "" pos:[115,122] width:55 type:#integer range:[0,65535,0]
        
        groupbox modify_group "Modify" pos:[5,150] width:175 height:90
            button id_offset_button "Offset IDs" pos:[15,170] width:95
            spinner offset_spinner "" pos:[115,173] width:55 type:#integer range:[-65534,65534,0]
            button id_swap_button "Swap IDs" pos:[15,195] width:155
            spinner swap_from_spinner "" pos:[15,220] width:55 type:#integer range:[0,65534,0]
            label with_label "with" pos:[82,220] width:25
            spinner swap_to_spinner "" pos:[115,220] width:55 type:#integer range:[0,65534,1]
        
        label status_label "INFO: Standying by..." pos:[5,245] width:175 height:15
        
        on id_generator_rollout moved pos do
            setIniSetting id_generator_ini "Dialog" "Position" (pos as string)
        
        on id_material_button pressed do
        (
            local object_list = id_generator.get_objects selection_checkbox.checked
            id_generator.set_ids_by_material object_list
            status_label.text = "INFO: IDs assigned by material"
        ) --end on
        
        on id_object_button pressed do
        (
            local object_list = id_generator.get_objects selection_checkbox.checked
            id_generator.set_ids_by_object object_list
            status_label.text = "INFO: IDs assigned by object"
        ) --end on
        
        on id_user_button pressed do
        (
            local object_list = id_generator.get_objects selection_checkbox.checked
            id_generator.set_ids_by_user object_list user_spinner.value
            message = "INFO: ID " + (user_spinner.value as string) + " assigned"
            status_label.text = message
        ) --end on
        
        on id_offset_button pressed do
        (
            local object_list = id_generator.get_objects selection_checkbox.checked
            id_generator.offset_ids object_list offset_spinner.value
            message = "INFO: IDs offset by " + (offset_spinner.value as string)
            status_label.text = message
        ) --end on
        
        on id_swap_button pressed do
        (
            local object_list = id_generator.get_objects selection_checkbox.checked
            id_generator.swap_ids object_list swap_from_spinner.value swap_to_spinner.value
            message = "INFO: ID " + (swap_from_spinner.value as string)
            message += " swapped with ID " + (swap_to_spinner.value as string)
            status_label.text = message
        ) --end on
    ) --end rollout
    
    id_generator_position = execute (getIniSetting id_generator_ini "Dialog" "Position")
	if id_generator_position == OK do id_generator_position = [100,100]
	if id_generator_position.x > sysinfo.desktopsize.x - 185 do
        id_generator_position.x = sysinfo.desktopsize.x - 190
	if id_generator_position.y > sysinfo.desktopsize.y - 320 do
        id_generator_position.y = sysinfo.desktopsize.y - 320
	if id_generator_position.x < 0 do id_generator_position.x = 0 
	if id_generator_position.y < 0 do id_generator_position.y = 0
    createDialog id_generator_rollout 185 260 id_generator_position.x id_generator_position.y
) --end local scope
